@@ -17,20 +17,19 @@ import copy
 import errno
 import os
 import os.path
-import tempfile
 
 import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
-from oslotest import base as test_base
 
 from ironic_lib import exception
+from ironic_lib.tests import base
 from ironic_lib import utils
 
 CONF = cfg.CONF
 
 
-class BareMetalUtilsTestCase(test_base.BaseTestCase):
+class BareMetalUtilsTestCase(base.IronicLibTestCase):
 
     def test_unlink(self):
         with mock.patch.object(os, "unlink", autospec=True) as unlink_mock:
@@ -45,101 +44,9 @@ class BareMetalUtilsTestCase(test_base.BaseTestCase):
             unlink_mock.assert_called_once_with("/fake/path")
 
 
-class ExecuteTestCase(test_base.BaseTestCase):
-
-    def test_retry_on_failure(self):
-        fd, tmpfilename = tempfile.mkstemp()
-        _, tmpfilename2 = tempfile.mkstemp()
-        try:
-            fp = os.fdopen(fd, 'w+')
-            fp.write('''#!/bin/sh
-# If stdin fails to get passed during one of the runs, make a note.
-if ! grep -q foo
-then
-    echo 'failure' > "$1"
-fi
-# If stdin has failed to get passed during this or a previous run, exit early.
-if grep failure "$1"
-then
-    exit 1
-fi
-runs="$(cat $1)"
-if [ -z "$runs" ]
-then
-    runs=0
-fi
-runs=$(($runs + 1))
-echo $runs > "$1"
-exit 1
-''')
-            fp.close()
-            os.chmod(tmpfilename, 0o755)
-            try:
-                self.assertRaises(processutils.ProcessExecutionError,
-                                  utils.execute,
-                                  tmpfilename, tmpfilename2, attempts=10,
-                                  process_input=b'foo',
-                                  delay_on_retry=False)
-            except OSError as e:
-                if e.errno == errno.EACCES:
-                    self.skipTest("Permissions error detected. "
-                                  "Are you running with a noexec /tmp?")
-                else:
-                    raise
-            fp = open(tmpfilename2, 'r')
-            runs = fp.read()
-            fp.close()
-            self.assertNotEqual(runs.strip(), 'failure', 'stdin did not '
-                                'always get passed '
-                                'correctly')
-            runs = int(runs.strip())
-            self.assertEqual(10, runs,
-                             'Ran %d times instead of 10.' % (runs,))
-        finally:
-            os.unlink(tmpfilename)
-            os.unlink(tmpfilename2)
-
-    def test_unknown_kwargs_raises_error(self):
-        self.assertRaises(processutils.UnknownArgumentError,
-                          utils.execute,
-                          '/usr/bin/env', 'true',
-                          this_is_not_a_valid_kwarg=True)
-
-    def test_check_exit_code_boolean(self):
-        utils.execute('/usr/bin/env', 'false', check_exit_code=False)
-        self.assertRaises(processutils.ProcessExecutionError,
-                          utils.execute,
-                          '/usr/bin/env', 'false', check_exit_code=True)
-
-    def test_no_retry_on_success(self):
-        fd, tmpfilename = tempfile.mkstemp()
-        _, tmpfilename2 = tempfile.mkstemp()
-        try:
-            fp = os.fdopen(fd, 'w+')
-            fp.write('''#!/bin/sh
-# If we've already run, bail out.
-grep -q foo "$1" && exit 1
-# Mark that we've run before.
-echo foo > "$1"
-# Check that stdin gets passed correctly.
-grep foo
-''')
-            fp.close()
-            os.chmod(tmpfilename, 0o755)
-            try:
-                utils.execute(tmpfilename,
-                              tmpfilename2,
-                              process_input=b'foo',
-                              attempts=2)
-            except OSError as e:
-                if e.errno == errno.EACCES:
-                    self.skipTest("Permissions error detected. "
-                                  "Are you running with a noexec /tmp?")
-                else:
-                    raise
-        finally:
-            os.unlink(tmpfilename)
-            os.unlink(tmpfilename2)
+class ExecuteTestCase(base.IronicLibTestCase):
+    # Allow calls to utils.execute() and related functions
+    block_execute = False
 
     @mock.patch.object(processutils, 'execute', autospec=True)
     @mock.patch.object(os.environ, 'copy', return_value={}, autospec=True)
@@ -221,7 +128,7 @@ grep foo
         self._test_execute_with_log_stdout(log_stdout=False)
 
 
-class MkfsTestCase(test_base.BaseTestCase):
+class MkfsTestCase(base.IronicLibTestCase):
 
     @mock.patch.object(utils, 'execute', autospec=True)
     def test_mkfs(self, execute_mock):
@@ -272,7 +179,7 @@ class MkfsTestCase(test_base.BaseTestCase):
                           'ext4', '/my/block/dev', 'ext4-vol')
 
 
-class IsHttpUrlTestCase(test_base.BaseTestCase):
+class IsHttpUrlTestCase(base.IronicLibTestCase):
 
     def test_is_http_url(self):
         self.assertTrue(utils.is_http_url('http://127.0.0.1'))
@@ -283,7 +190,7 @@ class IsHttpUrlTestCase(test_base.BaseTestCase):
         self.assertFalse(utils.is_http_url('11111111'))
 
 
-class ParseRootDeviceTestCase(test_base.BaseTestCase):
+class ParseRootDeviceTestCase(base.IronicLibTestCase):
 
     def test_parse_root_device_hints_without_operators(self):
         root_device = {
@@ -291,7 +198,7 @@ class ParseRootDeviceTestCase(test_base.BaseTestCase):
             'serial': 'foo-serial', 'vendor': 'foo VENDOR with space',
             'name': '/dev/sda', 'wwn_with_extension': '123456111',
             'wwn_vendor_extension': '111', 'rotational': True,
-            'hctl': '1:0:0:0'}
+            'hctl': '1:0:0:0', 'by_path': '/dev/disk/by-path/1:0:0:0'}
         result = utils.parse_root_device_hints(root_device)
         expected = {
             'wwn': 's== 123456', 'model': 's== foo%20model',
@@ -299,7 +206,8 @@ class ParseRootDeviceTestCase(test_base.BaseTestCase):
             'vendor': 's== foo%20vendor%20with%20space',
             'name': 's== /dev/sda', 'wwn_with_extension': 's== 123456111',
             'wwn_vendor_extension': 's== 111', 'rotational': True,
-            'hctl': 's== 1%3A0%3A0%3A0'}
+            'hctl': 's== 1%3A0%3A0%3A0',
+            'by_path': 's== /dev/disk/by-path/1%3A0%3A0%3A0'}
         self.assertEqual(expected, result)
 
     def test_parse_root_device_hints_with_operators(self):
@@ -309,13 +217,14 @@ class ParseRootDeviceTestCase(test_base.BaseTestCase):
             'name': '<or> /dev/sda <or> /dev/sdb',
             'wwn_with_extension': 's!= 123456111',
             'wwn_vendor_extension': 's== 111', 'rotational': True,
-            'hctl': 's== 1:0:0:0'}
+            'hctl': 's== 1:0:0:0', 'by_path': 's== /dev/disk/by-path/1:0:0:0'}
 
         # Validate strings being normalized
         expected = copy.deepcopy(root_device)
         expected['model'] = 's== foo%20model'
         expected['vendor'] = 's== foo%20vendor%20with%20space'
         expected['hctl'] = 's== 1%3A0%3A0%3A0'
+        expected['by_path'] = 's== /dev/disk/by-path/1%3A0%3A0%3A0'
 
         result = utils.parse_root_device_hints(root_device)
         # The hints already contain the operators, make sure we keep it
@@ -400,6 +309,10 @@ class ParseRootDeviceTestCase(test_base.BaseTestCase):
         self.assertRaises(ValueError, utils.parse_root_device_hints,
                           {'hctl': 123})
 
+    def test_parse_root_device_hints_invalid_by_path(self):
+        self.assertRaises(ValueError, utils.parse_root_device_hints,
+                          {'by_path': 123})
+
     def test_parse_root_device_hints_non_existent_hint(self):
         self.assertRaises(ValueError, utils.parse_root_device_hints,
                           {'non-existent': 'foo'})
@@ -486,7 +399,7 @@ class ParseRootDeviceTestCase(test_base.BaseTestCase):
             ValueError, utils._normalize_hint_expression, '', 'size')
 
 
-class MatchRootDeviceTestCase(test_base.BaseTestCase):
+class MatchRootDeviceTestCase(base.IronicLibTestCase):
 
     def setUp(self):
         super(MatchRootDeviceTestCase, self).setUp()
